function [ndiv, SyncClass, m, z] = AndrzejakChaos2021CoreIterationR2(c,G,nmax,ndiag,HiResndiv,maxm,epsnum,exactperiodtest)
% This source code allows you to reproduce the simulations presented in
%
% Andrzejak RG
% Chimeras confined in fractal boundaries in the complex plane
% Chaos 31, 053104 (2021); doi: 10.1063/5.0049631
%
% Please give a reference to this manuscript if you use this source code in
% your publications.
%
% This is the second release of this source code. The first release would not 
% have detected across pair chimeras and triplet states. 
% This is now fixed. See lines 161-179 and lines 187-209.
% Using the system parameters in our paper, these states were never found. 
% In fact, we assume that they do not exist. 
% Nonetheless, if you used the code applying other system parameters, you 
% should run some simulations again to assure that these states also do not
% exist in your settings.
% 
%
% Please read the general comments in AndrzejakChaos2021Example.m
%
% INPUT
%
% c:          Complex-valued parameter value c of the quadratic map.
% G:          Coupling matrix. Must be set for two populations of two maps each.
% nmax:       Maximal number of iterations.
% ndiag:      Every ndiag-th iteration, the dynamics is analysed. The loop is
%             interrupted if the maps have diverged, converged to a fixed point,
%             or the stopping criterion of 'exactperiodtest' is met.
% HiResndiv:  1: Determine ndiv with resolution of 1. (used in the manuscript)
%             0: Determine ndiv with resolution ndiag (faster, but less resolution)
% maxm:       The dynamics is tested for periodicities of up to maxm.
% epsnum:     Epsilon value used to check for synchronization and periodicity
% exactperiodtest:
% 1:    Stops the iteration if the periodicity is exact (and
%       not only up to epsnum).
% 0:    Do not use this stopping criterion
% WARNING THIS CRITERION WAS NOT USED IN OUR MANUSCRIPT. WE
% ADDED IT FOR THIS SOURCE CODE RELEASE. TO THE BEST OF OUR
% UNDERSTANDING, IT WORKS FINE. NONETHELESS, IF YOU WANT TO USE
% IT, YOU SHOULD TEST ITS CORRECTNESS.
%
% OUTPUT
%
% ndiv:       Number of iterations at which onset of divergence is detected.
%
% SyncClass:  Code indicating the asymptotic state of the dynamics
%             We use codes starting from 501 because the values of 1-500 are
%             used in other parts to encode the onset of divergence.
%   501: Divergence detected because values are NAN.
%   502: Divergence detected because values are INF (see comment below).
%   503: Full synchronization.
%   504: Within-pair synchronization.
%   505: Chimera.
%   506: Across-pair synchronization.
%   507: Triplet synchronization (never detected in our simulation).
%   508: Across-pair chimera (never detected in our simulation).
%   509: Full desynchronzation.
%
% m:        Period
%       Positive m values in 1:maxm indicate that all maps have period m
%       -1: Divergence detected, i.e. period not defined.
%       -2: Maps have different periods (see comment below).
%
% z:       Iterates of all four maps. Can be used to plot figure 1 in the
%          manuscript.



% Allocate space for iterates and set initial conditions
z=zeros(4,nmax);
z0 = 0;
z(1,1)=z0^2+c;
z(2,1)=z(1,1)^2+c;
z(3,1)=z(2,1)^2+c;
z(4,1)=z(3,1)^2+c;

% Set period m and outcome SyncClass to zero, implying undefined
m = 0;
SyncClass = 0;

for n = 2:nmax;
    % Core iteration of the four maps
    temp1 = z(:,n-1).^2+c;
    z(:,n)=temp1+sum(bsxfun(@minus,temp1.',temp1).*G,2);
    % Core iteration of the four maps done
    
    % Analysis of maps done every ndiag-th iteration
    if sum(n==[ndiag:ndiag:nmax])
        if sum(isnan(z(:,n)))
            % divergence detected because values are NAN
            SyncClass = 501;
            m=-1; % Period set to -1 for divergent dynamics
            break
        end
        if sum(isinf(z(:,n)))
            % divergence detected because values are INF
            % Never occurred in our simulations. Kept active for security.
            SyncClass = 502;
            m=-1; % Period set to -1 for divergent dynamics
            break
        end
        % Testing each of the four maps for period-m dynamics
        % Allocate help variable
        mIndividual = zeros(1,4);
        exactperiod = zeros(1,4);
        for mapindex = 1:4
            for testform = 1:min(n-1,maxm)
                % min is needed since maxm might be higher than the current iteration index
                if abs(z(mapindex,n)-z(mapindex,n-testform))<epsnum
                    mIndividual(mapindex)=testform;
                    if exactperiodtest
                        if abs(z(mapindex,n)-z(mapindex,n-testform))==0
                            exactperiod(mapindex) = 1;
                        else
                            exactperiod(mapindex) = 0;
                        end
                    end
                    break
                end
            end
        end
        if std(mIndividual)==0
            % All maps have the same period
            m=testform;
        else
            % Different maps have different periods.
            % This case happened very seldom. Visual inspection of the
            % iterates suggested that this was caused by very long transients,
            % and the detected periods are not yet the asymptotic ones. See
            % the manuscript for a more detailed discussion.
            m=-2; % -2 is a code used in this case, it is not a true period.
        end
        
        
        if maxpairwisediff(z(:,n),4)<epsnum;
            %AAAA, Full synchronization
            SyncClass = 503;
        else
            if (maxpairwisediff(z(1:2,n),2)<epsnum) && (maxpairwisediff(z(3:4,n),2)<epsnum)
                % AABB, within-pair synchronization
                SyncClass = 504;
            else
                if (maxpairwisediff(z(1:2,n),2)>epsnum) && (maxpairwisediff(z(3:4,n),2)<epsnum)
                    % ACBB, chimera state, 1st configuration
                    SyncClass = 505;
                else
                    if (maxpairwisediff(z(1:2,n),2)<epsnum) && (maxpairwisediff(z(3:4,n),2)>epsnum)
                        % AACB, chimera state, 2nd configuration
                        SyncClass = 505;
                    else
                        if (maxpairwisediff(z([1 3],n),2)<epsnum) && (maxpairwisediff(z([2 4],n),2)<epsnum)
                            %ABAB, across-pair synchronization, 1st configuration
                            SyncClass = 506;
                        else
                            if (maxpairwisediff(z([1 4],n),2)<epsnum) && (maxpairwisediff(z([2 3],n),2)<epsnum)
                                %ABBA, across-pair synchronization, 2nd configuration
                                SyncClass = 506;
                            else
                                if (maxpairwisediff(z([1 3],n),2)<epsnum) && (maxpairwisediff(z([2 4],n),2)>epsnum)
                                    %ABAC, across-pair chimera, 1st configuration
                                    SyncClass = 508;
                                else
                                    if (maxpairwisediff(z([1 4],n),2)<epsnum) && (maxpairwisediff(z([2 3],n),2)>epsnum)
                                        %ABCA, across-pair chimera, 2nd configuration
                                        SyncClass = 508;
                                    else
                                        if (maxpairwisediff(z([2 3],n),2)<epsnum) && (maxpairwisediff(z([1 4],n),2)>epsnum)
                                            %BAAC, across-pair chimera, 3rd configuration
                                            SyncClass = 508;
                                        else
                                            if (maxpairwisediff(z([2 4],n),2)<epsnum) && (maxpairwisediff(z([1 3],n),2)>epsnum)
                                                %BACA, across-pair chimera, 4th configuration
                                                SyncClass = 508;
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
        
        % Rule out that a chimera is in fact a triplet synchronization
        if SyncClass == 505
            if (maxpairwisediff(z([1 2 3],n),3)<epsnum)
                %AAAB, triplet synchronization, 1st configuration
                SyncClass = 507;
            else
                if (maxpairwisediff(z([1 2 4],n),3)<epsnum)
                    %AABA, triplet synchronization, 2nd configuration
                    SyncClass = 507;
                else
                    if (maxpairwisediff(z([1 3 4],n),3)<epsnum)
                        %ABAA, triplet synchronization, 3rd configuration
                        SyncClass = 507;
                    else
                        if (maxpairwisediff(z([2 3 4],n),3)<epsnum)
                            %BAAA, triplet synchronization, 4th configuration
                            SyncClass = 507;
                        else
                        end
                    end
                end
            end
        end
        if m==1
            % Iteration can be stopped since fixed point is detected.
            % fprintf('+');
            break
        end
        if exactperiodtest
            if sum(exactperiod)==4
                % Iteration can be stopped since the periodicity is exact (and
                % not only up to epsnum).
                % WARNING THIS CRITERION WAS NOT USED IN OUR MANUSCRIPT. WE
                % ADDED IT FOR THIS SOURCE CODE RELEASE. TO THE BEST OF OUR
                % UNDERSTANDING IT WORKS FINE. NONETHELESS, IF YOU WANT TO USE
                % IT, YOU SHOULD TEST ITS CORRECTNESS.
                % fprintf('-');
                break
            end
        end
    end
    if SyncClass==0
        % No synchronization detected
        if maxpairwisediff(z(:,n),4)>epsnum;
            %ABCD, Full desynchronzation
            SyncClass = 509;
        end
    end
end

if HiResndiv
    if SyncClass==501 || SyncClass==502
        % The maps diverged. ndiv is determined with resolution of 1.
        for n = 2:nmax;
            temp1 = z(:,n-1).^2+c;
            z(:,n)=temp1+sum(bsxfun(@minus,temp1.',temp1).*G,2);
            if max(abs(z(:,n)))>2
                break
            end
        end
    end
end

ndiv = n;
% ndiv is defined as the last iteration prior to breaking the loop.


    function [Mdist] = maxpairwisediff(z,k)
        
        MMdist = zeros(k,k);
        for i=1:k
            for j = i+1:k
                MMdist(i,j) = abs(z(i)-z(j));
            end
        end
        Mdist = max(max(MMdist));
    end

end