function AndrzejakChaos2021MainR2(ReMin,ReMax,RePix,ImMin,ImMax,ImPix,nmax,Cw,Ca,epsnum,exactperiodtest,ndiag,HiResndiv,maxm,verbose)

% This source code allows you to reproduce the simulations presented in
%
% Andrzejak RG
% Chimeras confined in fractal boundaries in the complex plane
% Chaos 31, 053104 (2021); doi: 10.1063/5.0049631
%
% Please give a reference to this manuscript if you use this source code in
% your publications.
%
% Please read the general comments in AndrzejakChaos2021Example.m
%
% INPUT
% ReMin,ReMax: Minimal and maximal value for real part of parameter c.
% ImMin,ImMax: Minimal and maximal value for imaginary part of parameter c.
% RePix,ImPix: Number of values placed from min to max at real and
%              imaginary part. Corresponds to pixels in the final display.
% nmax:        Maximal number of iterations.
% Cw:          Coupling within pairs.
% Ca:          Coupling across pairs.
% epsnum:      Epsilon value used to check for synchronization and periodicity.
% ndiag:       Every ndiag-th iteration, the dynamics is analysed. The loop is
%              interrupted if the maps have diverged, converged to a fixed point, 
%              or the stopping criterion of 'exactperiodtest' is met.
% HiResndiv:   1: Determine ndiv with resolution of 1. (used in the manuscript) 
%              0: Determine ndiv with resolution ndiag (faster, but less resolution)
% maxm:        The dynamics is tested for periodicities up to maxm.
% verbose:     Provide screen output to report about progress.
% exactperiodtest: 
            % 1:    Stops the iteration if the periodicity is exact (and
            %       not only up to epsnum).
            % 0:    Do not use this stopping criterion
            %
            % WARNING THIS CRITERION WAS NOT USED IN OUR MANUSCRIPT. WE 
            % ADDED IT FOR THIS SOURCE CODE RELEASE. TO THE BEST OF OUR 
            % UNDERSTANDING, IT WORKS FINE. NONETHELESS, IF YOU WANT TO USE 
            % IT, YOU SHOULD TEST ITS CORRECTNESS.


            
% Define connectivity matrix corresponding to two populations
G = ones(4,4)*Ca;
G(1:2,1:2)=Cw;
G(3:4,3:4)=Cw;

% Define ranges for parameter c;
ReRange=ReMin:(ReMax-ReMin)/(RePix-1):ReMax;
ImRange =ImMin:(ImMax-ImMin)/(ImPix-1):ImMax;


ndivMatrix=zeros(ImPix,RePix);
SyncClassMatrix=zeros(ImPix,RePix);
mMatrix = zeros(ImPix,RePix);
% Perfom main calculation in two nested for for-loops
for ReIndex = 1:RePix
    if verbose
        fprintf('Working on column %i of %i. Stopping criteria influence computation time per column.\n', ReIndex, RePix)
    end
    for ImIndex = 1:ImPix
        c = ReRange(ReIndex)+1i*ImRange(ImIndex);
        [ndivMatrix(ImIndex,ReIndex), SyncClassMatrix(ImIndex,ReIndex), mMatrix(ImIndex,ReIndex), ~] = ... 
            AndrzejakChaos2021CoreIteration(c,G,nmax,ndiag,HiResndiv,maxm,epsnum,exactperiodtest);
    end
end


% Visualization
% Rescale range of log10(ndiv) to range [0,500] linearly
% In the manuscript, we sometimes used piecewise linear scaling to enhance
% contrast
ndivMatrix = log10(ndivMatrix);
ndivMax = max(max(ndivMatrix));
ndivMin = min(min(ndivMatrix));
ndivMatrix = 500*(ndivMatrix-ndivMin)/(ndivMax-ndivMin);

% Combine ndiv and SyncClass
figure;
PlotndivMatrix = ndivMatrix;
PlotndivMatrix(SyncClassMatrix>502) = SyncClassMatrix(SyncClassMatrix>502);
imagesc(ReRange,ImRange,PlotndivMatrix,[1,509]);
set(gca,'YDir','normal')
% Define color map 
% Gray colors for divergent behavior.
colormapsync = bone(500);
% Non-gray colors for states of synchronization.
colormapsync(503,:)= [.298 .741 .929];   % light blue, full synchronization
colormapsync(504,:)= [.926 .690 .122];   % yellow, within-pair synchronization
colormapsync(505,:)= [.441 .162 .498];   % purple, chimera
colormapsync(506,:)= [.370 .537 .148];   % green, across-pair synchronization
colormapsync(507,:)= [1.00 .000 1.00];   % Magenta, triplet synchronization (not seen in paper)
colormapsync(508,:)= [.000 .000 1.00];   % Pure blue, across pair chimera (not seen in paper)
colormapsync(509,:)= [.758 .149 .216];   % Red, full desynchronization
% There is no deeper meaning of these specific color codes above. Adapt them to
% what you like more.
colormap(colormapsync);
xlabel('real part c');    
ylabel('imaginary part c');    
title('nmax combined with synchronization class');

% Combine ndiv and m
figure;
PlotmMatrix = ndivMatrix;
plotmaxm = 20; % maximal period to display. Rest casted in plot to plotmaxm.
for m = 1:plotmaxm-1
  PlotmMatrix(mMatrix==m) = 500+floor(10*log(m))+1;    
end
PlotmMatrix(mMatrix>=plotmaxm) = 500+floor(10*log(plotmaxm))+1;
imagesc(ReRange,ImRange,PlotmMatrix,[1,500+floor(10*log(plotmaxm))+1]);
set(gca,'YDir','normal')
colormapmp1 = bone(500);
colormapmp2 = jet(floor(10*log(plotmaxm))+1);
colormapm = [colormapmp1; colormapmp2];
colormap(colormapm);
xlabel('real part c');    
ylabel('imaginary part c');    
title('nmax combined with period m');

